@@ -4,18 +4,59 @@ import warnings
 
 import requests
 import yaml
-from urllib3.util.retry import Retry
+
+# Requests module historically (>=0.8.0) bundled its own urllib3,
+# however it may depend on how the module is installed, as some
+# distributions (e.g. RHEL-7) ship the same version of requests
+# module without bundled urllib3 and add a compat layer aliasing
+# requests.packages.urllib3 to non-bundled urllib3.
+# New versions (>=2.16) of the requests module stopped bundling
+# urllib3 and added a different backwards-compat layer to retain
+# requests.packages.* imports, however that layer is nontransparent
+# to static analyzers such as pylint, hence the disable=import-error
+# annotation even though the actual import still works.
+from requests.packages.urllib3.util.retry import Retry  # pylint: disable=import-error
+
+# It is preferred to return the data as immutable when possible
+try:
+    from frozendict import frozendict
+except ImportError:  # pragma: no cover
+    # If frozendict is not available, fall back to builtin dict
+    frozendict = dict  # pylint: disable=invalid-name
+try:
+    from frozenlist2 import frozenlist
+except ImportError:  # pragma: no cover
+    # If frozenlist is not available, fall back to builtin list
+    frozenlist = list  # pylint: disable=invalid-name
+
+
+def freeze(node):
+    """
+    Converts dict to frozendict and list to frozenlist.
+    """
+    if isinstance(node, list):
+        # Iterating using index instead of enumeration
+        # so we can replace the list items in place
+        for index in range(len(node)):  # pylint: disable=consider-using-enumerate
+            node[index] = freeze(node[index])
+        return frozenlist(node)
+    if isinstance(node, dict):
+        for key, value in node.items():
+            node[key] = freeze(value)
+        return frozendict(node)
+    return node
 
 
 def parsed(data, ext):
     """
     Parses a JSON or YAML string or a requests.Response object.
     """
+    is_response = isinstance(data, requests.Response)
     if ext.lower() == ".json":
-        return data.json() if isinstance(data, requests.Response) else json.loads(data)
-    return yaml.load(
-        data.text if isinstance(data, requests.Response) else data, yaml.SafeLoader
-    )
+        parsed_data = data.json() if is_response else json.loads(data)
+    else:
+        parsed_data = yaml.safe_load(data.text if is_response else data)
+    return freeze(parsed_data)
 
 
 def get_remote_data(url, session=None):
@@ -98,9 +139,8 @@ def load_schema():
     Returns:
         The cdn-definitions schema, coerced into a Python object.
     """
-    with open(
-        os.path.join(os.path.dirname(os.path.dirname(__file__)), "schema.json")
-    ) as schema:
+    schema_filename = os.path.join(os.path.dirname(__file__), "../schema.json")
+    with open(schema_filename) as schema:  # pylint: disable=unspecified-encoding
         return json.load(schema)
 
 
